package de.ruv.nvus.basis.client.ui.controller;

import java.util.Arrays;
import java.util.Comparator;
import java.util.List;

import org.eclipse.core.databinding.DataBindingContext;
import org.eclipse.core.databinding.beans.BeansObservables;
import org.eclipse.core.databinding.observable.list.WritableList;
import org.eclipse.core.databinding.observable.value.ComputedValue;
import org.eclipse.core.databinding.observable.value.IObservableValue;
import org.eclipse.riena.beans.common.Person;
import org.eclipse.riena.beans.common.PersonFactory;
import org.eclipse.riena.navigation.ISubModuleNode;
import org.eclipse.riena.navigation.ui.controllers.SubModuleController;
import org.eclipse.riena.ui.ridgets.AbstractCompositeRidget;
import org.eclipse.riena.ui.ridgets.IActionListener;
import org.eclipse.riena.ui.ridgets.IActionRidget;
import org.eclipse.riena.ui.ridgets.ICompositeTableRidget;
import org.eclipse.riena.ui.ridgets.ILabelRidget;
import org.eclipse.riena.ui.ridgets.IMultipleChoiceRidget;
import org.eclipse.riena.ui.ridgets.IRidget;
import org.eclipse.riena.ui.ridgets.IRowRidget;
import org.eclipse.riena.ui.ridgets.ISingleChoiceRidget;
import org.eclipse.riena.ui.ridgets.ITextRidget;

/**
 * Controller for the {@link CompositeTableSubModuleView} example.
 */
public class CompositeTableTest extends SubModuleController {
	static public final String LORE_IPSUM =
			" Lorem ipsum dolor sit amet, consectetuer sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum";

	public static class RowRidget extends AbstractCompositeRidget implements IRowRidget {
		private Person rowData;

		private static final String[] GENDER = { Person.FEMALE, Person.MALE };

		public void setData(Object rowData) {
			this.rowData = (Person)rowData;
		}

		@Override
		public void configureRidgets() {
			ITextRidget txtFirst = (ITextRidget)getRidget("first"); //$NON-NLS-1$
			txtFirst.bindToModel(rowData, Person.PROPERTY_FIRSTNAME);
			txtFirst.updateFromModel();

			ITextRidget txtLast = (ITextRidget)getRidget("last"); //$NON-NLS-1$
			txtLast.bindToModel(rowData, Person.PROPERTY_LASTNAME);
			txtLast.updateFromModel();

			ILabelRidget lblRidget = (ILabelRidget)getRidget("complete"); //$NON-NLS-1$
			lblRidget.setText(rowData.getFirstname() + " " + rowData.getLastname() + LORE_IPSUM);

			ISingleChoiceRidget gender = (ISingleChoiceRidget)getRidget("gender"); //$NON-NLS-1$
			gender.bindToModel(Arrays.asList(GENDER), (List<String>)null, rowData, Person.PROPERTY_GENDER);
			gender.updateFromModel();

			IMultipleChoiceRidget pets = (IMultipleChoiceRidget)getRidget("pets"); //$NON-NLS-1$
			pets.bindToModel(Arrays.asList(Person.Pets.values()), (List<String>)null, rowData, Person.PROPERTY_PETS);
			pets.updateFromModel();
		}
	}

	private final List<Person> input = PersonFactory.createPersonList();

	public CompositeTableTest() {
		this(null);
	}

	public CompositeTableTest(ISubModuleNode navigationNode) {
		super(navigationNode);
	}

	@Override
	public void configureRidgets() {
		final ICompositeTableRidget table = (ICompositeTableRidget)getRidget("table"); //$NON-NLS-1$
		final IActionRidget buttonAdd = (IActionRidget)getRidget("buttonAdd"); //$NON-NLS-1$
		final IActionRidget buttonDelete = (IActionRidget)getRidget("buttonDelete"); //$NON-NLS-1$
		final IActionRidget buttonDump = (IActionRidget)getRidget("buttonDump"); //$NON-NLS-1$

		table.bindToModel(new WritableList(input, Person.class), Person.class, RowRidget.class);
		table.updateFromModel();
		table.setComparator(0, new Comparator<Object>() {
			public int compare(Object o1, Object o2) {
				Person p1 = (Person)o1;
				Person p2 = (Person)o2;
				int result = p1.getLastname().compareTo(p2.getLastname());
				if (result == 0) {
					result = p1.getFirstname().compareTo(p2.getFirstname());
				}
				return result;
			}
		});
		table.setComparator(1, new Comparator<Object>() {
			public int compare(Object o1, Object o2) {
				Person p1 = (Person)o1;
				Person p2 = (Person)o2;
				return p1.getGender().compareTo(p2.getGender());
			}
		});
		table.setSortedColumn(0);

		buttonAdd.setText("&Add"); //$NON-NLS-1$
		buttonAdd.addListener(new IActionListener() {
			private int i = 0;

			public void callback() {
				i++;
				Person person = new Person("Doe #" + i, "John"); //$NON-NLS-1$ //$NON-NLS-2$
				person.setHasCat(true);
				input.add(person);
				table.updateFromModel();
				table.setSelection(person);
			}
		});

		buttonDelete.setText("&Delete"); //$NON-NLS-1$
		buttonDelete.addListener(new IActionListener() {
			public void callback() {
				Person person = (Person)table.getSingleSelectionObservable().getValue();
				input.remove(person);
				table.updateFromModel();
			}
		});

		buttonDump.setText("&Console dump"); //$NON-NLS-1$
		buttonDump.addListener(new IActionListener() {
			public void callback() {
				System.out.println("\nPersons:"); //$NON-NLS-1$
				for (Person p : input) {
					System.out.println(p);
				}
			}
		});

		final IObservableValue viewerSelection = table.getSingleSelectionObservable();
		IObservableValue hasSelection = new ComputedValue(Boolean.TYPE) {
			@Override
			protected Object calculate() {
				return Boolean.valueOf(viewerSelection.getValue() != null);
			}
		};
		DataBindingContext dbc = new DataBindingContext();
		bindEnablementToValue(dbc, buttonDelete, hasSelection);
	}

	// helping methods
	// ////////////////

	private void bindEnablementToValue(DataBindingContext dbc, IRidget ridget, IObservableValue value) {
		dbc.bindValue(BeansObservables.observeValue(ridget, IRidget.PROPERTY_ENABLED), value, null, null);
	}

}
